"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findFiles = exports.analyzeProject = void 0;
const config_1 = require("@previewjs/config");
const fs_extra_1 = __importDefault(require("fs-extra"));
const glob_1 = __importDefault(require("glob"));
const path_1 = __importDefault(require("path"));
const util_1 = require("util");
const caching_1 = require("./caching");
async function analyzeProject(workspace, options = {}) {
    const cacheFilePath = path_1.default.join((0, caching_1.getCacheDir)(workspace.rootDirPath), "components.json");
    useCache: if (!options.forceRefresh && (await fs_extra_1.default.pathExists(cacheFilePath))) {
        const stat = await fs_extra_1.default.stat(cacheFilePath);
        if (stat.mtimeMs < Date.now() - 24 * 3600 * 1000) {
            // We want to force refresh once a day at least.
            break useCache;
        }
        return {
            components: JSON.parse(await fs_extra_1.default.readFile(cacheFilePath, "utf8")),
            cached: true,
        };
    }
    const components = await analyzeProjectCore(workspace);
    await fs_extra_1.default.mkdirp(path_1.default.dirname(cacheFilePath));
    await fs_extra_1.default.writeFile(cacheFilePath, JSON.stringify(components));
    return { components, cached: false };
}
exports.analyzeProject = analyzeProject;
async function analyzeProjectCore(workspace) {
    const absoluteFilePaths = await findFiles(workspace.rootDirPath, "**/*.@(js|jsx|ts|tsx|svelte|vue)");
    const components = {};
    const found = await workspace.frameworkPlugin.detectComponents(workspace.typeAnalyzer, absoluteFilePaths);
    for (const component of found) {
        const filePath = path_1.default.relative(workspace.rootDirPath, component.absoluteFilePath);
        const fileComponents = (components[filePath] || (components[filePath] = []));
        fileComponents.push({
            componentName: component.name,
            exported: component.exported,
        });
    }
    return components;
}
async function findFiles(rootDirPath, pattern) {
    const config = (await (0, config_1.readConfig)(rootDirPath));
    const files = await (0, util_1.promisify)(glob_1.default)(pattern, {
        ignore: ["**/node_modules/**", ...(config.exclude || [])],
        cwd: rootDirPath,
        nodir: true,
        absolute: true,
        follow: false,
    });
    // Note: in some cases, presumably because of yarn using link
    // for faster node_modules, glob may return files in the parent
    // directory. We filter them out here.
    return files.filter((f) => f.startsWith(rootDirPath.replace(/\\/g, "/") + "/"));
}
exports.findFiles = findFiles;
