import { TypeAnalyzer } from "@previewjs/type-analyzer";
import { Reader } from "@previewjs/vfs";
import express from "express";
import * as vite from "vite";
import { ProjectAnalysis } from "./analyze-project";
import { PersistedStateManager } from "./persisted-state";
import { FrameworkPlugin } from "./plugins/framework";
import { ApiRouter } from "./router";
export type { ProjectAnalysis } from "./analyze-project";
export { generateComponentId } from "./component-id";
export type { PersistedStateManager } from "./persisted-state";
export type { Component, ComponentAnalysis, FrameworkPlugin, FrameworkPluginFactory, } from "./plugins/framework";
export { loadPreviewEnv } from "./preview-env";
export type { PreviewEnvironment, SetupPreviewEnvironment, } from "./preview-env";
export declare function createWorkspace({ versionCode, rootDirPath, reader, frameworkPlugin, logLevel, middlewares, onReady, persistedStateManager, }: {
    versionCode: string;
    rootDirPath: string;
    middlewares: express.RequestHandler[];
    frameworkPlugin: FrameworkPlugin;
    logLevel: vite.LogLevel;
    reader: Reader;
    persistedStateManager?: PersistedStateManager;
    onReady?(options: {
        router: ApiRouter;
        workspace: Workspace;
    }): Promise<void>;
}): Promise<Workspace | null>;
/**
 * Returns the absolute directory path of the closest ancestor containing node_modules.
 */
export declare function findWorkspaceRoot(absoluteFilePath: string): string | null;
export interface Workspace {
    rootDirPath: string;
    reader: Reader;
    typeAnalyzer: TypeAnalyzer;
    frameworkPlugin: FrameworkPlugin;
    preview: {
        start(allocatePort?: () => Promise<number>): Promise<Preview>;
    };
    components: {
        list(options?: {
            forceRefresh?: boolean;
        }): Promise<ProjectAnalysis>;
    };
    dispose(): Promise<void>;
}
export interface Preview {
    url(): string;
    stop(options?: {
        onceUnused?: boolean;
    }): Promise<void>;
}
