"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiRouter = void 0;
class ApiRouter {
    constructor() {
        this.handlers = new Map();
    }
    async handle(path, request, expressRequest, expressResponse) {
        const handler = this.handlers.get(path);
        if (!handler) {
            return {
                kind: "error",
                message: `No handler is registered for path '${path}'`,
            };
        }
        try {
            const response = await handler(request, expressRequest, expressResponse);
            return {
                kind: "success",
                response,
            };
        }
        catch (e) {
            console.error(`Handler ${path} failed`, e);
            return {
                kind: "error",
                message: e.message,
            };
        }
    }
    onRequest(endpoint, handler) {
        if (this.handlers.has(endpoint.path)) {
            throw new Error(`Multiple handlers registered for endpoint '${endpoint.path}'`);
        }
        this.handlers.set(endpoint.path, handler);
    }
}
exports.ApiRouter = ApiRouter;
