"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportToplevelPlugin = void 0;
const acorn_1 = require("acorn");
const path_1 = __importDefault(require("path"));
const jsExtensions = new Set([".js", ".jsx", ".ts", ".tsx"]);
function exportToplevelPlugin() {
    return {
        name: "previewjs:export-toplevel",
        enforce: "post",
        transform: async function (code, id) {
            if (id.indexOf(`/node_modules/`) !== -1) {
                return null;
            }
            const extension = path_1.default.extname(id);
            if (!jsExtensions.has(extension)) {
                return null;
            }
            const topLevelEntityNames = findTopLevelEntityNames(id, code);
            return `${code};
          export {
          ${topLevelEntityNames
                .map((c) => `${c} as __previewjs__${c},`)
                .join("")}
        }`;
        },
    };
}
exports.exportToplevelPlugin = exportToplevelPlugin;
function findTopLevelEntityNames(filePath, source) {
    var _a, _b, _c;
    let parsed;
    try {
        parsed = acorn_1.Parser.parse(source, {
            ecmaVersion: "latest",
            sourceType: "module",
        });
    }
    catch (e) {
        throw new Error(`Unable to parse ${filePath} with Acorn: ${e}`);
    }
    const topLevelEntityNames = [];
    // Note: acorn doesn't provide detailed typings.
    for (const statement of parsed.body || []) {
        if (statement.type === "VariableDeclaration") {
            for (const declaration of statement.declarations) {
                if (declaration.type === "VariableDeclarator") {
                    addIfIdentifier(topLevelEntityNames, declaration.id);
                }
            }
        }
        if (statement.type === "FunctionDeclaration") {
            addIfIdentifier(topLevelEntityNames, statement.id);
        }
        if (statement.type === "ClassDeclaration") {
            addIfIdentifier(topLevelEntityNames, statement.id);
        }
        if (statement.type === "ExportDefaultDeclaration") {
            if ((_a = statement.declaration) === null || _a === void 0 ? void 0 : _a.id) {
                addIfIdentifier(topLevelEntityNames, statement.declaration.id);
            }
        }
        if (statement.type === "ExportNamedDeclaration") {
            if ((_b = statement.declaration) === null || _b === void 0 ? void 0 : _b.id) {
                addIfIdentifier(topLevelEntityNames, statement.declaration.id);
            }
            if ((_c = statement.declaration) === null || _c === void 0 ? void 0 : _c.declarations) {
                for (const declarator of statement.declaration.declarations) {
                    addIfIdentifier(topLevelEntityNames, declarator.id);
                }
            }
        }
    }
    return topLevelEntityNames;
}
function addIfIdentifier(array, id) {
    if (id && id.type === "Identifier") {
        const name = id.name;
        if (name.endsWith("_default")) {
            return;
        }
        array.push(name);
    }
}
